@@ -4,10 +4,12 @@ import logging
 import xml.etree.ElementTree as ET
 
 from urlparse import urlparse
+from namenode import Namenode
 
 log = logging.getLogger(__name__)
 
 class HDFSConfig(object):
+    use_trash = False
 
     @classmethod
     def get_config_from_env(cls):
@@ -25,8 +27,9 @@ class HDFSConfig(object):
             # if config exists in hdfs - it's HA config, update configs
             configs = tmp_config
 
-        if configs is None:
+        if not configs:
             raise Exception("No config found in %s nor in %s" % (core_path, hdfs_path))
+
         return configs
 
 
@@ -41,13 +44,20 @@ class HDFSConfig(object):
 
     @classmethod
     def read_core_config(cls, core_site_path):
+        config = []
         for property in cls.read_hadoop_config(core_site_path):
             if property.findall('name')[0].text == 'fs.defaultFS':
                 parse_result = urlparse(property.findall('value')[0].text)
                 log.debug("Got namenode '%s' from %s" % (parse_result.geturl(), core_site_path))
-                return [{"namenode": parse_result.hostname,
-                         "port": parse_result.port}]
-        return []
+
+                config.append({"namenode": parse_result.hostname,
+                               "port": parse_result.port if parse_result.port
+                                                         else Namenode.DEFAULT_PORT})
+
+            if property.findall('name')[0].text == 'fs.trash.interval':
+                cls.use_trash = True
+
+        return config
 
     @classmethod
     def read_hdfs_config(cls, hdfs_site_path):
@@ -57,7 +67,11 @@ class HDFSConfig(object):
                 parse_result = urlparse("//" + property.findall('value')[0].text)
                 log.debug("Got namenode '%s' from %s" % (parse_result.geturl(), hdfs_site_path))
                 configs.append({"namenode": parse_result.hostname,
-                                "port": parse_result.port})
+                                "port": parse_result.port if parse_result.port
+                                                          else Namenode.DEFAULT_PORT})
+
+            if property.findall('name')[0].text == 'fs.trash.interval':
+                cls.use_trash = True
 
         return configs
 
@@ -87,5 +101,4 @@ class HDFSConfig(object):
                 if tmp_config:
                     # if there is hdfs-site data available return it
                     return tmp_config
-
             return configs
